// nanoMat.cpp, a really small C++ matrix library
// author: Justin Domke
// contact: (last name of author)@cs.umd.edu
// history:
// first version written June 13-14, 2008
// updated June 16, 2008 to fix a bug, and add the assign_* add_* and sub_*
//                             methods for speed
// updated August 4, 2008 to fix a bug in col_eq, and add col_eq and row_eq
//                             for arguments of doubles
// updated August 13, 2008 to use templates-- in case you don't like doubles
//
// see http://www.cs.umd.edu/~domke/code/nanoMat/ for documentation

// you can define these flags if you roll that way
//#define nanoMatDebug 1 // for safety/error checking; comment out for faster and unsafe(er)
//#define nanoMatDalloc 1 // this trick makes MSVC 6 faster-- turn off for g++

// Class definition here-- no header file.  If this appals you, this is
// probably not the matrix library for you...
#include <iostream>

#define d(i,j) data[i + M*j]

//typedef double(*oneInOneOut)(double); // function pointers for mapping

template <class T>
class nanoMat{
    public:
        nanoMat();
        nanoMat(const mxArray *matlab_array);
        nanoMat(T *data, int M, int N);      
        nanoMat(int M, int N);                    // allocate memory for mat
        nanoMat(int M_in, int N_in, T init); // same and assign all to init
        nanoMat(const nanoMat& src);
        ~nanoMat();
        
        void operator=(const nanoMat& B);  // does a "deep" copy, safe but sometimes wasteful

        void printMe();
  
        T& operator() (int i);
        T& operator() (int i, int j);
        
        // first, all the pairwise operations on two matrices...
        nanoMat operator*(const nanoMat& A); // matrix mult
        nanoMat operator%(const nanoMat& A); // elementwise mult
        nanoMat operator/(const nanoMat& A); // elementwise div
        nanoMat operator+(const nanoMat& A); // elementwise +
        nanoMat operator-(const nanoMat& A); // elementwise
        
        // now, all those operations vs. a double
        // (two funcs each so the double can be on either side);
        nanoMat operator*(T num);
        nanoMat operator/(T num);
        nanoMat operator+(T num);
        nanoMat operator-(T num);
        
        // not const so that you are allowed to do, e.g. A += A;
        // (not that I care about that-- the complier insists...)
        void operator%=(const nanoMat& B);
        void operator+=(const nanoMat& B);
        void operator-=(const nanoMat& B);
        void operator/=(const nanoMat& B);

        void operator*=(T num);
        void operator/=(T num);
        void operator+=(T num);
        void operator-=(T num);
        
        nanoMat Tr(); // transpose
        nanoMat sum(int dim);
        nanoMat repmat(int i, int j);
        
        nanoMat row(int i);
        nanoMat col(int i);
        nanoMat colD(int i); // dangerous column
        
        void row_eq(int i, nanoMat& B);
        void col_eq(int i, nanoMat& B);
        void row_eq(int i, T d);
        void col_eq(int i, T d);
        
        void norm_col(int i);
        void norm_row(int i);
        
        nanoMat max(int dim);
        
        //nanoMat map(oneInOneOut fun);
        //void map_eq(oneInOneOut fun);
        
        // this functions are uglier, faster versions of things
        // these add no functionality, but allow optimization
        // I suggest you ignore these until the program is debugged.  If
        // it is still insufficently fast, use these to eliminate
        // memory allocation
        void assign(nanoMat& B);
        void assign_colD(nanoMat& B, int i);
        void assign_mult(nanoMat& A, nanoMat& B);
        void add_mult(nanoMat& A, nanoMat& B);
        void add_mult(nanoMat& A, T d);
        void sub_mult(nanoMat& A, nanoMat& B);
        void sub_mult(nanoMat& A, T d);
        void assign_sum(nanoMat& A, int dim);
        void vecTD(); // destructively transposes a vector (swaps the dimensions)
        
        T* data;
        int M; // LENGTH of columns (# of rows)
        int N; // LENGTH of rows    (# of cols)
        int *dalloc; // flag needed to know if should destroy memory when
                     // class dies off
};

template <class T>
nanoMat<T> operator*(T num, nanoMat<T>& A);
template <class T>
nanoMat<T> operator/(T num, nanoMat<T>& A);
template <class T>
nanoMat<T> operator+(T num, nanoMat<T>& A);
template <class T>
nanoMat<T> operator-(T num, nanoMat<T>& A);

using namespace std;

template <class T>
nanoMat<T>::nanoMat(){
    //printf("default constructor");
    data = NULL;
    M = 0;
    N = 0;
    //dalloc = 0;
    dalloc = new int(0);
}

template <class T>
nanoMat<T>::nanoMat(const mxArray *matlab_array){
    // copy (by reference) the matlab array
    data = (T*)mxGetData(matlab_array);
    M  = mxGetM(matlab_array);
    N  = mxGetN(matlab_array);
    //dalloc = 0;
    dalloc = new int(0);
    //printf("matlab constructor: %d x %d\n", M, N);
}

template <class T>
nanoMat<T>::nanoMat(T *data, int M, int N){
    this->data = data;
    this->M    = M;
    this->N    = N;
    //dalloc = 0;
    dalloc = new int(0);
}

template <class T>
nanoMat<T>::nanoMat(int M_in, int N_in){
    //printf("empty constructor: %d x %d\n", M_in, N_in);
    M    = M_in;
    N    = N_in;
    data = new T[M*N]; // UNINITIALIZED
    //dalloc = 1;
    dalloc = new int(1);
}

template <class T>
nanoMat<T>::nanoMat(int M_in, int N_in, T init){
    //printf("empty constructor: %d x %d\n", M_in, N_in);
    M    = M_in;
    N    = N_in;
    data = new T[M*N];
    for(int i=0; i<M*N; i++)
        data[i] = init;
    //dalloc = 1;
    dalloc = new int(1);
}

template <class T>
nanoMat<T>::nanoMat(const nanoMat& B){
    //printf("copy constructor\n");
    N = B.N;
    M = B.M;
    dalloc = new int(1);
    if( *(B.dalloc)==2){ // danger flag lets you take the array with out copy
        data = B.data;
        *(B.dalloc) = 0;
        return;
    }
    data = new T[M*N];
    for(int i=0; i<M*N; i++)
        data[i] = B.data[i];
    //memcpy(data,B.data,M*N * sizeof(T)); // MUCH faster
    return;
}

template <class T>
nanoMat<T>::~nanoMat(){
    if(*dalloc==1 || *dalloc==2)
        delete[] data;
    delete dalloc;
    //printf("destruction...\n");  // useful for debugging
}

template <class T>
void nanoMat<T>::operator=(const nanoMat& B){
    if(*dalloc==1){
        delete[] data;
    }
    N = B.N;
    M = B.M;
    *dalloc = 1;
    if( *(B.dalloc)==2){ // danger flag lets you take the array with out copy
        data = B.data;
        *(B.dalloc) = 0;
        return;
    }
    data = new T[M*N];
    for(int i=0; i<M*N; i++)
            data[i] = B.data[i];
    //memcpy(data,B.data,M*N * sizeof(T)); // MUCH faster
    return;
}

/*
template <class T>
void nanoMat<T>::printMe(){
    printf("%d x %d array:\n", M, N);
    for(int i=0; i<M; i++){
        for(int j=0; j<N; j++){
            printf("% 9.4f ", d(i,j) );
        }
        printf("\n");
    }
    return;
}*/
// specialized for doubles and ints and ADvars
template<>
void nanoMat<double>::printMe(){
    mexPrintf("%d x %d array:\n", M, N);
    for(int i=0; i<M; i++){
        for(int j=0; j<N; j++){
            mexPrintf("% 9.4f ", d(i,j) );
        }
        mexPrintf("\n");
    }
    return;
}

template<>
void nanoMat<int>::printMe(){
    printf("%d x %d array:\n", M, N);
    for(int i=0; i<M; i++){
        for(int j=0; j<N; j++){
            printf("% d ", d(i,j) );
        }
        printf("\n");
    }
    return;
}

/*
template<>
void nanoMat<ADvar>::printMe(){
    printf("%d x %d array:\n", M, N);
    for(int i=0; i<M; i++){
        for(int j=0; j<N; j++){
            printf("% 9.4f ", d(i,j).val() );
        }
        printf("\n");
    }
    return;
}
*/

template <class T>
inline T& nanoMat<T>::operator()(int i){
    #ifdef nanoMatDebug
    if(i< 0){
        printf("illegal index: (i), i<0 %d<0 \n", i);
        return data[0];
    }
    if(i>=M*N){
        printf("illegal index: (i), i>=M*N  %d>=%d M=%d  N=%d \n", i, M*N, M, N);
        return data[0];
        }
    #endif
    return data[i];
}

template <class T>
inline T& nanoMat<T>::operator()(int i, int j){
    #ifdef nanoMatDebug
    if(i< 0){
        printf("illegal index: (i,j), i<0  \n");
        return data[0];
    }if(i>=M){
        printf("illegal index: (i,j), i>=M \n");
        return data[0];
    }if(j< 0){
        printf("illegal index: (i,j), j<0  \n");
        return data[0];
    }if(j>=N){
        printf("illegal index: (i,j), j>=N  %d>=%d \n", j, N);
        return data[0];
        }
    #endif
    return data[i + M*j];
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator*(const nanoMat<T>& A){ // elementwise *
    //printf("operator*\n");
    nanoMat<T> C = nanoMat<T>(M,A.N);
    #ifdef nanoMatDebug
    if(N != A.M){
        printf("illegal dimensions for matrix multiply (%d vs. %d)\n",this->N,A.M);
        return C;
    }
    #endif
    // this naive matrix mult algorithm is O(N^3)
    for(int i=0; i<C.M; i++)
        for(int j=0; j<C.N; j++){
            C(i,j) = 0;
            for(int k=0; k<this->N; k++)
                C(i,j) += (*this)(i,k)*A.data[k+j*A.M];
                //C(i,j) += (*this)(i,k)*A(k,j);
        }
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator%(const nanoMat<T>& A){ // elementwise *
    nanoMat<T> C = nanoMat<T>(*this);
    C %= A;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator/(const nanoMat<T>& A){ // elementwise /
    nanoMat<T> C = nanoMat<T>(*this);
    C /= A;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}
 
template <class T>
inline nanoMat<T> nanoMat<T>::operator+(const nanoMat<T>& A){ // elementwise +
    nanoMat<T> C = nanoMat<T>(*this);
    C += A;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator-(const nanoMat<T>& A){ // elementwise -
    nanoMat<T> C = nanoMat<T>(*this);
    C -= A;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}


//WAKEUP
template <class T>
inline nanoMat<T> nanoMat<T>::operator*(T num){
    nanoMat<T> C = nanoMat<T>(*this);
    C *= num;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    //mexPrintf("C.dalloc %d  this->alloc: %d  KOSTAS SUCKS \n", *(C.dalloc), *(this->dalloc));
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator/(T num){
    nanoMat<T> C = nanoMat<T>(*this);
    C /= num;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator+(T num){
    nanoMat<T> C = nanoMat<T>(*this);
    C += num;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::operator-(T num){
    nanoMat<T> C = nanoMat<T>(*this);
    C -= num;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> operator*(T num, nanoMat<T>& A){
    nanoMat<T> C = nanoMat<T>(A);
    C *= num;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> operator/(T num, nanoMat<T>& A){
    nanoMat<T> C = nanoMat<T>(A.M,A.N,num);
    C /= A;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> operator+(T num, nanoMat<T>& A){
    nanoMat<T> C = nanoMat<T>(A);
    C += num;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> operator-(T num, nanoMat<T>& A){
    nanoMat<T> C = nanoMat<T>(A.M,A.N,num);
    C -= A;
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline void nanoMat<T>::operator%=(const nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(this->M != B.M || this->N != B.N){
        printf("%%= called with illegal dimensions\n");
        return;
    }
    #endif
    for(int i=0; i<M*N; i++)
        data[i] *= B.data[i];
//        data[i] *= B(i);
}

template <class T>
inline void nanoMat<T>::operator+=(const nanoMat<T>& B){
    //mexPrintf("+= at your service.  this->dalloc: %d, B.dalloc=%d\n", *(this->dalloc), *(B.dalloc));
    #ifdef nanoMatDebug
    if(this->M != B.M || this->N != B.N){
        printf("+= called with illegal dimensions\n");
        return;
    }
    #endif
    for(int i=0; i<M*N; i++)
        data[i] += B.data[i];
//        data[i] += B(i);
}

template <class T>
inline void nanoMat<T>::operator-=(const nanoMat<T>& B){
    //mexPrintf("-= at your service.  this->dalloc: %d, B.dalloc=%d\n", *(this->dalloc), *(B.dalloc));
    #ifdef nanoMatDebug
    if(this->M != B.M || this->N != B.N){
        printf("-= called with illegal dimensions\n");
        return;
    }
    #endif
    for(int i=0; i<M*N; i++)
        data[i] -= B.data[i];
        //data[i] -= B(i);
}

template <class T>
inline void nanoMat<T>::operator/=(const nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(this->M != B.M || this->N != B.N){
        printf("/= called with illegal dimensions\n");
        return;
    }
    #endif
    for(int i=0; i<M*N; i++)
        data[i] /= B.data[i];
//        data[i] /= B(i);
}

template <class T>
inline void nanoMat<T>::operator*=(T num){
    for(int i=0; i<M*N; i++)
        data[i] *= num;
}

template <class T>
inline void nanoMat<T>::operator/=(T num){
    for(int i=0; i<M*N; i++)
        data[i] /= num;
}

template <class T>
inline void nanoMat<T>::operator+=(T num){
    for(int i=0; i<M*N; i++)
        data[i] += num;
}

template <class T>
inline void nanoMat<T>::operator-=(T num){
    for(int i=0; i<M*N; i++)
        data[i] -= num;
}

template <class T>
inline nanoMat<T> nanoMat<T>::Tr(){ // transpose
    nanoMat<T> C = nanoMat<T>(N,M); // notice reverse dims
    for(int i=0; i<M; i++)
        for(int j=0; j<N; j++)
            C(j,i) = d(i,j);
    #ifdef nanoMatDalloc
    *(C.dalloc) = 2; // live dangerously
    #endif
    return C;
}

template <class T>
inline nanoMat<T> nanoMat<T>::sum(int dim){
    #ifdef nanoMatDebug
        if( !( dim==1 || dim==2))
            printf("illegal sum dim\n");
    #endif
    if( dim==1){ // flatten columns
        nanoMat<T> B = nanoMat<T>(1,N,0.0);
        for(int i=0; i<N; i++)
            for(int j=0; j<M; j++)
                B(0,i) += d(j,i);
        #ifdef nanoMatDalloc
        *(B.dalloc) = 2; // live dangerously
        #endif
        return B;
    } else if( dim==2){ // flatten rows
        nanoMat<T> B = nanoMat<T>(M,1,0.0);
        for(int i=0; i<N; i++)
            for(int j=0; j<M; j++)
                B(j,0) += d(j,i);
        #ifdef nanoMatDalloc
        *(B.dalloc) = 2; // live dangerously
        #endif
        return B;
    } else{
        printf("sum with no legal dim.\n");
    }
}


template <class T>
inline nanoMat<T> nanoMat<T>::repmat(int I, int J){
    nanoMat B = nanoMat(M*I,N*J);
    for(int i=0; i<I; i++)
        for(int j=0; j<J; j++)
            for(int m=0; m<M; m++)
                for(int n=0; n<N; n++)
                    B(i*M+m,j*N+n)=d(m,n);
    #ifdef nanoMatDalloc
    *(B.dalloc) = 2; // live dangerously
    #endif
    return B;
}

template <class T>
inline nanoMat<T> nanoMat<T>::row(int i){
    nanoMat<T> B = nanoMat<T>(1,N);
    for(int j=0; j<N; j++)
        B(j) = d(i,j);
    #ifdef nanoMatDalloc
    *(B.dalloc) = 2; // live dangerously
    #endif
    return B;
}

template <class T>
inline nanoMat<T> nanoMat<T>::col(int i){
    nanoMat<T> B = nanoMat<T>(M,1);
    for(int j=0; j<M; j++)
        B(j) = d(j,i);
    #ifdef nanoMatDalloc
    *(B.dalloc) = 2; // live dangerously
    #endif
    return B;
}

template <class T>
inline nanoMat<T> nanoMat<T>::colD(int i){ // dangerous column-- gives you just a reference inside
    T* newdata = data + i*M;
    nanoMat<T> B = nanoMat<T>(newdata,M,1);
    *(B.dalloc) = 0; // don't deallocate someone else's data
    return B;
}
    

template <class T>
inline void nanoMat<T>::row_eq(int i, nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(i<0 || i >= M){
        printf("Tried row_eq with illegal row index (i=%d M=%d)\n",i,M);
        return;
    }
    if(B.M != 1){
        printf("row_eq can only assign a matrix that is 1 x N\n");
        return;
    }
    if(B.N != N){
        printf("row_eq needs length of vector same size as matrix\n");
        return;
        }
    #endif
    for( int j=0; j<N; j++){
        d(i,j) = B(j);
    }
    return;
}

template <class T>
inline void nanoMat<T>::col_eq(int i, nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(i<0 || i >= N){
        printf("Tried col_eq with illegal col index (i=%d N=%d)\n",i,N);
        return;
    }
    if(B.N != 1){
        printf("col_eq can only assign a matrix that is M x 1\n");
        return;
    }
    if(B.M != M){
        printf("col_eq needs length of vector same size as matrix\n");
        return;
    }
    #endif
    for( int j=0; j<M; j++){
        d(j,i) = B(j);
    }
    return;
}

template <class T>
inline void nanoMat<T>::row_eq(int i, T d){
    #ifdef nanoMatDebug
    if(i<0 || i >= M){
        printf("Tried row_eq with illegal row index (i=%d M=%d)\n",i,M);
        return;
    }
    #endif
    for( int j=0; j<N; j++){
        d(i,j) = d;
    }
    return;
}

template <class T>
inline void nanoMat<T>::col_eq(int i, T d){
    #ifdef nanoMatDebug
    if(i<0 || i >= N){
        printf("Tried col_eq with illegal col index (i=%d N=%d)\n",i,N);
        return;
    }
    #endif
    for( int j=0; j<M; j++){
        d(j,i) = d;
    }
    return;
}

template <class T>
inline void nanoMat<T>::norm_col(int i){
    T mysum = 0.0;
    int j;
    for( j=0; j<M; j++)
        mysum += d(j,i);
    for( j=0; j<M; j++)
        d(j,i)  /= mysum;
}
//        void norm_row(int i);


//inline nanoMat nanoMat::map(oneInOneOut fun){
//    nanoMat B = nanoMat(*this);
//    B.map_eq(fun);
//    if(!nanoMatDebug) B.dalloc = 2; // live dangerously
//    return B;
//}


//inline void nanoMat::map_eq(oneInOneOut fun){
//    // version of map that doesn't need to make a copy
//    for(int i=0; i<M*N; i++)
//        data[i] = fun(data[i]);
//}

template <class T>
inline nanoMat<T> nanoMat<T>::max(int dim){
    int i,j;
    if(dim==1){
        nanoMat<T> B = nanoMat<T>(1,N);
        for( j=0; j<N; j++)
            B(j) = d(0,j);
        for(j=0; j<N; j++)
            for( i=0; i<M; i++)
                if(d(i,j) > B(j))
                    B(j) = d(i,j);
        #ifdef nanoMatDalloc
        *(B.dalloc) = 2; // live dangerously
        #endif
        return B;
    } else if(dim==2){
        nanoMat<T> B = nanoMat<T>(M,1);
        for( i=0; i<M; i++)
            B(i) = d(0,1);
        for(i=0; i<M; i++)
            for( j=0; j<N; j++)
                if(d(i,j) > B(i))
                    B(i) = d(i,j);
        #ifdef nanoMatDalloc
        *(B.dalloc) = 2; // live dangerously
        #endif
        return B;
    }
}

// does a fast copy of B to this
// totally avoids memory allocation
template <class T>
inline void nanoMat<T>::assign(nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(M!=B.M || N!=B.N){
        printf("dimensions do not match in assign\n");
        return;
    }
    #endif
    for(int i=0; i<M*N; i++)
        B.data[i] = data[i];
    //memcpy(data,B.data,M*N*sizeof(double));
}

template <class T>
inline void nanoMat<T>::assign_colD(nanoMat<T>& B,int i){
    #ifdef nanoMatDebug
    if(this->M != B.M || this->N != 1 || i >= B.N){
        printf("dimensions do not match for assign_colD\n");
        return;
    }
    if(i<0 || i>=B.N){
        printf("illegal column for assign_colD\n");
        return;
        }
    #endif
    if(*dalloc==1)
        delete[] data;
    data = B.data + i*B.M;
    *dalloc = 0; // don't deallocate someone else's data
    return;
}

template <class T>
inline void nanoMat<T>::assign_mult(nanoMat<T>& A, nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(this->M != A.M || this->N != B.N || A.N != B.M){
        printf("dimensions do not match for assign_mult\n");
        return;
    }
    #endif
    for(int i=0; i<this->M; i++)
        for(int j=0; j<this->N; j++){
            d(i,j) = 0;
            for(int k=0; k<A.N; k++)
                d(i,j) += A(i,k)*B(k,j);
        }
    return;
}

template <class T>
inline void nanoMat<T>::add_mult(nanoMat<T>& A, nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(this->M != A.M || this->N != B.N || A.N != B.M){
        printf("dimensions do not match for add_mult\n");
        return;
    }
    #endif
    for(int i=0; i<this->M; i++)
        for(int j=0; j<this->N; j++)
            for(int k=0; k<A.N; k++)
                d(i,j) += A(i,k)*B(k,j);
    return;    
}

template <class T>
inline void nanoMat<T>::sub_mult(nanoMat<T>& A, nanoMat<T>& B){
    #ifdef nanoMatDebug
    if(this->M != A.M || this->N != B.N || A.N != B.M){
        printf("dimensions do not match for add_mult\n");
        return;
    }
    #endif
    for(int i=0; i<this->M; i++)
        for(int j=0; j<this->N; j++)
            for(int k=0; k<A.N; k++)
                d(i,j) -= A(i,k)*B(k,j);
    return;    
}

template <class T>
inline void nanoMat<T>::add_mult(nanoMat<T>& A, T d){
    #ifdef nanoMatDebug
    if(this->M != A.M || this->N != A.N){
        printf("dimensions do not match for add_mult\n");
        return;
    }
    #endif
    int max = this->M*this->N;
    for(int i=0; i<max; i++)
        data[i] += A(i)*d;
    return;    
}

template <class T>
inline void nanoMat<T>::sub_mult(nanoMat<T>& A, T d){
    #ifdef nanoMatDebug
    if(this->M != A.M || this->N != A.N){
        printf("dimensions do not match for add_mult\n");
        return;
    }
    #endif
    int max = this->M*this->N;
    for(int i=0; i<max; i++)
        data[i] -= A(i)*d;
    return;    
}

template <class T>
inline void nanoMat<T>::assign_sum(nanoMat<T>& A, int dim){
    if( dim==1){ // flatten columns
        #ifdef nanoMatDebug
        if (this->M != 1 || this->N != A.N){
            printf("dimensions illegal in assign_sum\n");
            return;
        }
        #endif
        for(int i=0; i<A.N; i++){
            d(0,i) = 0;
            for(int j=0; j<A.M; j++)
                d(0,i) += A(j,i);
        }
        return;
    } else if( dim==2){ // flatten rows
        #ifdef nanoMatDebug
        if (this->M != A.M || this->N != 1){
            printf("dimensions illegal in assign_sum\n");
            return;
        }
        #endif
        for(int j=0; j<A.M; j++){
            d(j,0) = 0;
            for(int i=0; i<A.N; i++)        
                d(j,0) += A(j,i);
        }
        return;
    }
}

template <class T>
inline void nanoMat<T>::vecTD(){ // destructively transposes a vector (swaps the dimensions)
    #ifdef nanoMatDebug
    if(this->M != 1 && this->N != 1){
        printf("vecTD transposing a matrix that isn't a vector\n");
        return;
    }
    #endif
    int tmpN = this->M;
    this->M = this->N;
    this->N = tmpN;
}
